package com.nokia.task;

import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.SftpException;
import com.nokia.common.exception.MyRuntimeException;
import com.nokia.common.psql.PsqlUtil;
import com.nokia.common.ssh.SSHUtil;
import com.nokia.config.TaskConfig;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

@Data
@Component
@Slf4j
public class SyncTask {
    /**
     * 上次扫描到的文件
     */
    private String lastFilename;
    private SSHUtil sshUtil;
    private final JdbcTemplate jdbcTemplate;
    private final TaskConfig taskConfig;

    public SyncTask(JdbcTemplate jdbcTemplate, TaskConfig taskConfig) {
        this.jdbcTemplate = jdbcTemplate;
        this.taskConfig = taskConfig;
    }

    /**
     * 同步top用户信息定时任务
     *
     */
    @XxlJob("syncTopUser")
    public void syncTopUser() {
        try {
            CompletableFuture.runAsync(() -> {
                try {
                    singleTask();
                } catch (InterruptedException e) {
                    log.error("线程中断: {}", e.getMessage(), e);
                    XxlJobHelper.log("线程中断: {}", e.getMessage(), e);
                    Thread.currentThread().interrupt();
                    throw new MyRuntimeException(e);
                } catch (Exception e) {
                    throw new MyRuntimeException(e);
                }
            }).get(5, TimeUnit.MINUTES);
        } catch (InterruptedException e) {
            log.error("线程中断: {}", e.getMessage(), e);
            XxlJobHelper.log("线程中断: {}", e.getMessage(), e);
            Thread.currentThread().interrupt();
            XxlJobHelper.handleFail(e.getMessage());
            rollback();
        } catch (Exception e) {
            log.error("发生异常了: {}", e.getMessage(), e);
            XxlJobHelper.log("发生异常了: {}", e.getMessage(), e);
            XxlJobHelper.handleFail(e.getMessage());
            rollback();
        } finally {
            try {
                sshUtil.disconnect();
            } catch (IOException e) {
                log.error("发生异常了: {}", e.getMessage(), e);
                XxlJobHelper.log("发生异常了: {}", e.getMessage(), e);
                XxlJobHelper.handleFail(e.getMessage());
            }
        }
    }

    /**
     * 单一任务
     *
     */
    public void singleTask() throws JSchException, IOException, SftpException, InterruptedException {
        // 创建文件夹
        Files.createDirectories(Paths.get(taskConfig.getDownloadDir()));
        sshUtil = new SSHUtil(taskConfig.getSshHost(), taskConfig.getSshPort(), taskConfig.getSshUsername(),
                taskConfig.getSshPassword());
        // 获取文件列表
        List<String> list = sshUtil.ls(taskConfig.getSourceDir());
        if (CollectionUtils.isEmpty(list)) {
            log.error("没有文件");
            XxlJobHelper.log("没有文件");
            XxlJobHelper.handleFail("没有文件");
            return;
        }

        String filename = list.get(list.size() - 1);
        log.info("同步的文件: {}", filename);
        XxlJobHelper.log("同步的文件: {}", filename);
        if (filename.equals(lastFilename)) {
            throw new MyRuntimeException("没有新文件");
        }
        download(filename);
        distinct(filename);
        importData();
        lastFilename = filename;
    }

    /**
     * 下载文件
     *
     * @param filename 文件名
     */
    public void download(String filename) throws JSchException, IOException, SftpException {
        log.info("下载文件: {}", filename);
        XxlJobHelper.log("下载文件: {}", filename);
        String src = taskConfig.getSourceDir() + filename;
        String dst = taskConfig.getDownloadDir() + filename;
        sshUtil.get(src, dst);
    }

    /**
     * 去重
     *
     * @param filename 文件名
     */
    public void distinct(String filename) throws IOException {
        log.info("文件 {} 去重", filename);
        XxlJobHelper.log("文件 {} 去重", filename);
        String inputFilePath = taskConfig.getDownloadDir() + filename;
        Path inputPath = Paths.get(inputFilePath);
        try (CSVParser parser = CSVFormat.DEFAULT.builder().build()
                .parse(new InputStreamReader(Files.newInputStream(inputPath), "gbk"));
             OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(
                     Paths.get(taskConfig.getDistinctFilename())), StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
            Map<String, CSVRecord> csvRecordMap = new HashMap<>();
            // 按loginName去重
            for (CSVRecord t : parser) {
                if (t.getRecordNumber() == 1) {
                    continue;
                }
                csvRecordMap.put(t.get(4), t);
            }
            log.info("{} 条数据", csvRecordMap.size());
            XxlJobHelper.log("{} 条数据", csvRecordMap.size());
            if (CollectionUtils.isEmpty(csvRecordMap)) {
                throw new MyRuntimeException("数据为空");
            }
            Map<String, Map<String, Object>> cityMap = new HashMap<>();
            Map<Object, List<Map<String, Object>>> areaMap = new HashMap<>();
            getAreaMap(cityMap, areaMap);
            matchArea(printer, csvRecordMap, cityMap, areaMap);
            log.info("文件 {} 去重完成", filename);
            XxlJobHelper.log("文件 {} 去重完成", filename);
            // 删除本地源文件
            Files.deleteIfExists(inputPath);
            log.info("删除本地源文件 {}", filename);
            XxlJobHelper.log("删除本地源文件 {}", filename);
        }
    }

    /**
     * 得到区域地图
     *
     * @param cityMap 城市地图
     * @param areaMap 区域地图
     */
    private void getAreaMap(Map<String, Map<String, Object>> cityMap, Map<Object, List<Map<String, Object>>> areaMap) {
        // 查询所有地区
        String sql = "select * from sqmdb_rpt.acl_area order by type_code, area_id";
        List<Map<String, Object>> areas = jdbcTemplate.queryForList(sql);
        for (Map<String, Object> t : areas) {
            if (t.get("type_code").equals(2)) {
                cityMap.put((String) t.get("area_name"), t);
                areaMap.putIfAbsent(t.get("area_id"), new ArrayList<>());
                for (Map<String, Object> tt : areas) {
                    if (tt.get("type_code").equals(3) && tt.get("parent_id").equals(t.get("area_id"))) {
                        areaMap.get(t.get("area_id")).add(tt);
                    }
                }
            }
        }
    }

    /**
     * 匹配区域
     *
     * @param printer       打印机
     * @param csvRecordMap  csv记录地图
     * @param cityMap       城市地图
     * @param areaMap       区域地图
     */
    private void matchArea(CSVPrinter printer, Map<String, CSVRecord> csvRecordMap,
                           Map<String, Map<String, Object>> cityMap, Map<Object, List<Map<String, Object>>> areaMap)
            throws IOException {
        for (CSVRecord t : csvRecordMap.values()) {
            String orgId = t.get(0);
            String orgName = t.get(1);
            String userId = t.get(2);
            String userName = t.get(3);
            String loginName = t.get(4);
            String phone = t.get(5);
            String employeeCode = t.get(6);
            Integer provinceId = -1;
            Object cityId = -1;
            Object areaId = null;
            for (Map.Entry<String, Map<String, Object>> tt : cityMap.entrySet()) {
                // 匹配地市
                if (orgName.contains(tt.getKey())) {
                    Map<String, Object> area = tt.getValue();
                    cityId = area.get("area_id");
                    List<Map<String, Object>> areaList = areaMap.get(area.get("area_id"));
                    for (Map<String, Object> ttt : areaList) {
                        // 匹配区县
                        if (orgName.contains((CharSequence) ttt.get("area_name"))) {
                            areaId = ttt.get("area_id");
                            break;
                        }
                    }
                    break;
                }
            }
            printer.printRecord(loginName, orgId, orgName, userId, userName, phone, employeeCode, provinceId,
                    cityId, areaId);
        }
    }

    /**
     * 导入数据
     *
     */
    public void importData() throws IOException, InterruptedException {
        String sql = "drop table if exists sqmdb_rpt.acl_top_user_bak";
        log.info(sql);
        XxlJobHelper.log(sql);
        jdbcTemplate.execute(sql);
        sql = "create table sqmdb_rpt.acl_top_user_bak as select * from sqmdb_rpt.acl_top_user";
        log.info(sql);
        XxlJobHelper.log(sql);
        jdbcTemplate.execute(sql);
        sql = "truncate table sqmdb_rpt.acl_top_user";
        log.info(sql);
        XxlJobHelper.log(sql);
        jdbcTemplate.execute(sql);
        PsqlUtil.copyCsv(taskConfig.getImportScript(), taskConfig.getDbHost(), taskConfig.getDbPort(),
                taskConfig.getDbUsername(), taskConfig.getDbPassword(), taskConfig.getDbName(), taskConfig.getDbTable(),
                taskConfig.getDistinctFilename(), taskConfig.getMinInsertCount());
    }

    /**
     * 回滚
     */
    public void rollback() {
        String sql = "select exists (select from pg_tables where schemaname = 'sqmdb_rpt' and tablename = 'acl_top_user_bak')";
        Boolean backupExists = jdbcTemplate.queryForObject(sql, Boolean.class);
        log.warn(sql);
        XxlJobHelper.log(sql);
        if (Boolean.TRUE.equals(backupExists)) {
            sql = "truncate table sqmdb_rpt.acl_top_user";
            log.warn(sql);
            XxlJobHelper.log(sql);
            jdbcTemplate.execute(sql);
            sql = "insert into sqmdb_rpt.acl_top_user\n"
                    + "(login_name, org_id, org_name, user_id, user_name, phone, employee_code, province_id, city_id, area_id)\n"
                    + "select * from sqmdb_rpt.acl_top_user_bak";
            log.warn(sql);
            XxlJobHelper.log(sql);
            jdbcTemplate.execute(sql);
        }
    }
}
